function [parts, part_scales, part_score, image_data, superpixel_adjmat, ...
    superpixel_affinity, superpixel_sup_image] = ...
    DetectSymmetricParts_SingleImage(img_filename, classifier_params, recompute_on, ...
    felz_k, min_sup_affinity, min_shape_affinity, single_superpixel_parts_on, only_single_parts_on, num_sups)

    disp('Detecting symmetric parts ......');
    
    if (nargin < 3 || isempty(recompute_on))
        recompute_on = false;
    end
    
    if (nargin < 4 || isempty(felz_k))
        felz_k = 5;
    end
    
    if (nargin < 5 || isempty(min_sup_affinity))
        min_sup_affinity = 0.1;
    end
    
    if (nargin < 6 || isempty(min_shape_affinity))
        min_shape_affinity = 0.1;
    end
    
    if (nargin < 7 || isempty(single_superpixel_parts_on))
        single_superpixel_parts_on = false;
    end
    
    if (nargin < 8 || isempty(only_single_parts_on))
        only_single_parts_on = false;
    end
    
    if (nargin < 9 || isempty(num_sups))
        num_sups = [25, 50, 100, 200];
    end

    if (only_single_parts_on)
        num_sups = [5, 10, 20, 25, 30, 40, 50, 100, 200];
    end
    
    superpixel_sup_image = cell(numel(num_sups),1);
    superpixel_adjmat = cell(numel(num_sups),1);
    superpixel_affinity = cell(numel(num_sups),1);
    
    img = im2double(imread(img_filename));
    selected_features_idx = [1:100, 122:149];
    parts = [];
    part_scales = [];
    part_score = [];
    
    parts_filename = [img_filename(1:end-4),'_parts_felz',num2str(felz_k), ...
        '_minsup',num2str(min_sup_affinity), '_minshape', num2str(min_shape_affinity), ...
        '_',num2str(single_superpixel_parts_on), '_', num2str(only_single_parts_on), ...
        '_numsups_',strrep(num2str(num_sups),' ', '_'), '.mat'];
    
    if (exist(parts_filename, 'file') && ~recompute_on)
       load(parts_filename);
       return;
    end
    
    % Compute Pb
    pb_file = [img_filename(1:end-4),'_pb.mat'];
    if (~exist(pb_file, 'file') || recompute_on)
        disp('Extracting egdes using Pb');
        [pb, theta, tmap] = pbCGTG(img);
        save(pb_file, 'pb', 'theta', 'tmap');
    else
        load(pb_file);
    end
    
    image_data_file = [img_filename(1:end-4),'_image_data.mat'];
    if (~exist(image_data_file, 'file') || recompute_on)
        image_data = PrecomputeImageData(img, pb, theta);
        save(image_data_file, 'image_data');
    else
        load(image_data_file);
    end
        
    % Compute Ncuts superpixels
    for cur_num_sups = num_sups
        disp('Computing superpixels');
        superpixels_file = [img_filename(1:end-4),'_num_sups_',num2str(cur_num_sups),'.seg'];
        
        if (~exist(superpixels_file, 'file') || recompute_on)
            disp([num2str(cur_num_sups), ' superpixels']);
            sup_image = Superpixels_Ncuts_Pb(img, cur_num_sups, image_data_file);
            writeSeg(sup_image, superpixels_file);
        end            
    end
    
    % Compute the features
    disp('Clustering superpixels into parts');
    scale_idx = 0;
    for cur_num_sups = num_sups
        
        scale_idx = scale_idx + 1;
        superpixels_file = [img_filename(1:end-4),'_num_sups_',num2str(cur_num_sups),'.seg'];
        
        sup_image = readSeg(superpixels_file);
        
        superpixel_sup_image{scale_idx} = sup_image;
        affinity_features_file = [img_filename(1:end-4),'_num_sups_',num2str(cur_num_sups),'_affinity_features.mat'];

        if (~only_single_parts_on)
            if (recompute_on || ~exist(affinity_features_file, 'file'))
                [pairwise_features, adjmat, support_stats] = ...
                    SuperpixelAffinityFeatures(img, image_data, [], sup_image);
                save(affinity_features_file, 'sup_image', 'pairwise_features', 'adjmat', 'support_stats');
            else
                load(affinity_features_file);
            end
        
            features = pairwise_features(:,:,selected_features_idx);

            affinity = SuperpixelAffinityGetTrainedAffinity(classifier_params, adjmat, features);

            superpixel_adjmat{scale_idx} = adjmat;
            superpixel_affinity{scale_idx} = affinity;
        
            weights = 1 ./ (affinity + eps);
            weights(affinity == 0) = 0;
            [labels, component_affinity] = SuperpixelAffinityCluster(weights, adjmat, felz_k, ...
                image_data, sup_image, classifier_params, selected_features_idx, min_sup_affinity);
        else
            labels = (1:max(sup_image(:)))';
            component_affinity = zeros(size(labels));
        end
        
        if (single_superpixel_parts_on)
            
            single_sup_parts_file = [img_filename(1:end-4),'_num_sups_',num2str(cur_num_sups),'_single_sup_parts.mat'];
            
            if (recompute_on || ~exist(single_sup_parts_file, 'file'))
                single_sup_shape_affinity = SingleSuperpixelAffinity(sup_image, image_data, classifier_params, selected_features_idx);
                save(single_sup_parts_file, 'single_sup_shape_affinity');
            else
                load(single_sup_parts_file);
            end
        end

        num_labels = max(labels);
        parts_img = zeros(size(sup_image));
        num_parts = 0;
        
        for cur_label = 1:num_labels
            sup_idx = find(labels == cur_label);

            if ((numel(sup_idx) > 1  && ~only_single_parts_on) || ...
                (single_superpixel_parts_on && numel(sup_idx) == 1 && single_sup_shape_affinity(sup_idx) > min_shape_affinity))
                num_parts = num_parts + 1;

                part_BW = PartFromSuperpixels(sup_image, sup_idx);
                parts(:,:,end+1) = part_BW;
                part_scales(end+1) = scale_idx;
                
                if (numel(sup_idx) > 1)
                    part_score(end+1) = component_affinity(sup_idx(1));
                elseif (single_superpixel_parts_on)
                    part_score(end+1) = single_sup_shape_affinity(sup_idx);
                end
                for sup = sup_idx'
                    parts_img(sup_image == sup) = num_parts;
                end
            end
        end
    end
    
    parts = parts(:,:,2:end);
    save(parts_filename);
