function [seg_distance_closest, seg_distance_intersection, is_projected_inside, junction_point, dist1, dist2] = LineSegmentDistance(line_seg1, line_seg2)

% LineSegmentDistance - Computes the distance between two segments
%
% [seg_distance_closest, seg_distance_intersection, is_projected_inside] = LineSegmentDistance(line_seg1, line_seg2)
% Given two line segments, compute the distance between them.
% seg_distance_closest is the closest distance between any
% two points on the two segments. seg_distance_intersection is the sum of
% distances of the intersection point of the two segments (0 if the
% intersection point is inside both segments).
%
% Arguments:
% line_seg1, line_seg2 - two 4x1 vectors representing the two line segments
%
% Output:
% seg_distance_closest - a scalar representing the closest distance between segments
% seg_distance_intersection - a scalar representing the intersection
% distance between the two segments
% is_projected_inside - boolean value which is true if any of the endpoints
% of one segment project inside the other.
% junction_point - 2x1 vector storing the point where the two lines intersect
% dist1,dist2 - of lines to their intersection

    % Check if the two points of one segment fall on either side of the
    % other segment
    t1 = ((line_seg2(3) - line_seg2(1))*(line_seg1(2) - line_seg2(2)) - ...
        (line_seg2(4) - line_seg2(2))*(line_seg1(1) - line_seg2(1))) / ...
        ((line_seg2(4) - line_seg2(2))*(line_seg1(3) - line_seg1(1)) - ...
        (line_seg2(3) - line_seg2(1))*(line_seg1(4) - line_seg1(2)));
    t2 = ((line_seg1(3) - line_seg1(1))*(line_seg1(2) - line_seg2(2)) - ...
        (line_seg1(4) - line_seg1(2))*(line_seg1(1) - line_seg2(1))) / ...
        ((line_seg2(4) - line_seg2(2))*(line_seg1(3) - line_seg1(1)) - ...
        (line_seg2(3) - line_seg2(1))*(line_seg1(4) - line_seg1(2)));

    junction_point = line_seg1(1:2) + t1*(line_seg1(3:4)-line_seg1(1:2));
    
    if (t1 > 0 && t1 < 1 && t2 > 0 && t2 < 1)
    
        % The segments intersect.
        seg_distance_closest = 0;
        seg_distance_intersection = 0;
        is_projected_inside = true;
    else
    
        % The segments don't intersect. Compute the distance based on endpoints
        % projection
        [dist12, junction_point1, inside1] = PointToLineDist_MEX(line_seg2, reshape(line_seg1, [2,2]));
        [dist21, junction_point2, inside2] = PointToLineDist_MEX(line_seg1, reshape(line_seg2, [2,2]));
        seg_distance_closest = min([dist12;dist21]);

        dist1 = PointToLineDist_MEX(line_seg1, junction_point);
        dist2 = PointToLineDist_MEX(line_seg2, junction_point);
        seg_distance_intersection =  dist1 + dist2;
                   
        is_projected_inside = any([inside1; inside2]);
    end    
    