% SuperpixelAffinityCluster - Cluster superpixels based on their affinties
% with a method similar to the Felz. and Hut. method
%
% labels = SuperpixelAffinityCluster(affinity)
% Clusters elements based on their affinities. First, sort the affinities
% in decreasing order. Start merging elements with high affinity. Merge 
% over an edge e only if affinity(e) > k*min(min_affinity(A), min_affinity(B))
% where A and B are the components being merged by the edge, 
% min_affinity(A) is the minimum affinity between elements in A and
% min_affinity(B) is the minimum affinity between elements in B.
% Assign a cluster index to each element and returns it in "labels".
%
% 
function [labels, component_affinity] = SuperpixelAffinityCluster(weights, adjmat, k, image_data, sup_image, classifier_params, selected_features_idx, affinity_thresh)

    [sup1, sup2] = find(tril(adjmat, -1));
    [sorted_weights, sorted_idx] = sort(weights(sub2ind(size(adjmat), sup1, sup2)), 'ascend');
    
    num_sups = size(adjmat, 1);
    labels = (1:num_sups)';
    max_weights = zeros(size(labels));
    component_affinity = ones(size(labels));
%     component_affinity = SingleSuperpixelAffinity(sup_image, image_data, classifier_params, selected_features_idx);
    component_size = ones(size(labels));
    
    num_edges = numel(sup1);
    
    % Merge edges in decreasing order of affinity
    for i = 1:num_edges
        cur_sup1 = sup1(sorted_idx(i));
        cur_sup2 = sup2(sorted_idx(i));
        
        label1 = labels(cur_sup1);
        label2 = labels(cur_sup2);
        
        if (label1 == label2)
            continue;
        end
        
        cur_max_weight = min(max_weights(cur_sup1) + k/component_size(cur_sup1), ...
            max_weights(cur_sup2) + k/component_size(cur_sup2));
        cur_weight = weights(cur_sup1, cur_sup2);
        
        if (cur_weight <= cur_max_weight)
            
            elements1 = labels == label1;
            elements2 = labels == label2;
            
            new_label = min(label1, label2);
            all_elements = elements1 | elements2;
            
            elements1 = find(elements1);
            elements2 = find(elements2);
            feature = SuperpixelGroupFeature(image_data, sup_image, elements1, elements2, false);
            cur_affinity = SuperpixelRegionAffinityGetTrainedAffinity(feature(selected_features_idx)', classifier_params);
            
%             [ err, predictions ] = svmclassify(feature(selected_features_idx(1:100))', 1, classifier_params.svm_model);
%             predictions = [predictions, 1];
%             cur_affinity = 1 / (1 + exp(-predictions * classifier_params.logistic_shape));
            
            if (cur_affinity > affinity_thresh)
                labels(all_elements) = new_label;
                max_weights(all_elements) = cur_weight;
                component_size(all_elements) = sum(all_elements);
                component_affinity(all_elements) = cur_affinity;
            end
        end
    end
    
    unique_labels = unique(labels(:));
    new_labels = zeros(size(labels));
    num_labels = numel(unique_labels);

    for label = 1:num_labels
        idx = (labels == unique_labels(label));
        new_labels(idx) = label;
    end
    labels = new_labels;
    