% SuperpixelAffinityColorHistogram - Compute pairwise superpixel
% affinities based on color histograms
%
% color_hist_affinities = SuperpixelAffinityColorHistogram(img, seg_image, adjmat)
% Extracts color histograms for every superpixel. Compares the histograms
% of adjacent superpixels and returns the affinities.
%
% Arguments:
%   img - MxNx3 input image
%   seg_image - MxN superpixel segmentation storing superpixel indeces in
%   each pixel (0 if on the boundary or background).
%   adjmat - num_sups x num_sups matrix where num_sups = max(seg_image(:))
%   is the number of superpixels. 1 indicates that the corresponding
%   superpixels are neighbouring.
%
% Returns:
%   color_hist_affinities - num_sups x num_sups matrix of superpixel
%   affinities. Except for the diagonal, only entries corresponding to
%   adjacent superpixels are filled.
%
function color_hist_affinities = SuperpixelAffinityColorHistogram(img, seg_image, adjmat)

    color_hist_affinities = eye(size(adjmat));
    color_hist_affinities = repmat(color_hist_affinities,[1,1,27]);
    
    [sup1, sup2] = find(tril(adjmat, -1));
    num_sup_edges = numel(sup1);
    
    K = max(seg_image(:));
    num_bins = 10;
    img_r = img(:,:,1);
    img_g = img(:,:,2);
    img_b = img(:,:,3);
    hsv_img = rgb2hsv(img);
    img_h = hsv_img(:,:,1);
    img_s = hsv_img(:,:,2);
    img_v = hsv_img(:,:,3);
    color_hist = zeros(K, 3*num_bins);
    mean_color = zeros(K,3);
    var_color = zeros(K,3);
    mean_color_hsv = zeros(K,3);
    var_color_hsv = zeros(K,3);
    cos_hs = cos(img_h * 2 * pi) .* img_s;
    sin_hs = sin(img_h * 2 * pi) .* img_s;
    for i = 1:K
        BW = seg_image == i;
        color_hist(i, :) = RegionColorHistogram(BW, img_h, img_s, img_v, num_bins);
        mean_r = mean(img_r(BW));
        mean_g = mean(img_g(BW));
        mean_b = mean(img_b(BW));
        var_r = var(img_r(BW));
        var_g = var(img_g(BW));
        var_b = var(img_b(BW));

        mean_cos = mean(cos_hs(BW));
        mean_sin = mean(sin_hs(BW));
        mean_v = mean(img_v(BW));
        var_cos = var(cos_hs(BW));
        var_sin = var(sin_hs(BW));
        var_v = var(img_v(BW));

        mean_color(i,:) = [mean_r, mean_g, mean_b];
        var_color(i,:) = [var_r, var_g, var_b];

        mean_color_hsv(i,:) = [mean_cos, mean_sin, mean_v];
        var_color_hsv(i,:) = [var_cos, var_sin, var_v];
    end

    for sup_edge = 1:num_sup_edges
        cur_affinity_h = AffinityColorHistogram(color_hist(sup1(sup_edge),1:num_bins), color_hist(sup2(sup_edge), 1:num_bins));
        cur_affinity_s = AffinityColorHistogram(color_hist(sup1(sup_edge),num_bins+(1:num_bins)), color_hist(sup2(sup_edge), num_bins+(1:num_bins)));
        cur_affinity_v = AffinityColorHistogram(color_hist(sup1(sup_edge),2*num_bins+(1:num_bins)), color_hist(sup2(sup_edge), 2*num_bins+(1:num_bins)));
        cumsum_hist1 = cumsum(color_hist(sup1(sup_edge),2*num_bins+(1:num_bins)));
        cumsum_hist2 = cumsum(color_hist(sup2(sup_edge),2*num_bins+(1:num_bins)));
        dark1 = cumsum_hist1(3) >= 0.8;
        dark2 = cumsum_hist2(3) >= 0.8;
        is_bright = double(~dark1 & ~dark2);
        similar_brightness = double(dark1==dark2);

        dark1 = 1-mean_color_hsv(sup1(sup_edge), 3);
        dark2 = 1-mean_color_hsv(sup2(sup_edge), 3);
        is_bright = (1-dark1)*(1-dark2);
        similar_brightness = cur_affinity_v;
%         feature = [is_bright*cur_affinity_h, is_bright*cur_affinity_s, (1-is_bright)*similar_brightness];
        
%         feature = [abs(mean_color(sup1(sup_edge),:) - mean_color(sup2(sup_edge),:)), ...
%             abs(var_color(sup1(sup_edge),:) - var_color(sup2(sup_edge),:)), ...
%             var_color(sup1(sup_edge),:), var_color(sup2(sup_edge),:)];

        feature = [abs(mean_color(sup1(sup_edge),:) - mean_color(sup2(sup_edge),:)), ...
            abs(var_color(sup1(sup_edge),:) - var_color(sup2(sup_edge),:)), ...
            var_color(sup1(sup_edge),:), var_color(sup2(sup_edge),:), ...
            abs(mean_color_hsv(sup1(sup_edge),:) - mean_color_hsv(sup2(sup_edge),:)), ...
            abs(var_color_hsv(sup1(sup_edge),:) - var_color_hsv(sup2(sup_edge),:)), ...
            var_color_hsv(sup1(sup_edge),:), var_color_hsv(sup2(sup_edge),:), ...
            is_bright*cur_affinity_h, is_bright*cur_affinity_s, (1-is_bright)*similar_brightness];

        color_hist_affinities(sup1(sup_edge), sup2(sup_edge),:) = feature;
        color_hist_affinities(sup2(sup_edge), sup1(sup_edge),:) = feature;
        
%         color_hist_affinities(sup1(sup_edge), sup2(sup_edge),:) = [color_hist(sup1(i), :), color_hist(sup2(i), :)];
%         color_hist_affinities(sup2(sup_edge), sup1(sup_edge),:) = [color_hist(sup1(i), :), color_hist(sup2(i), :)];
    end
    

    