% SuperpixelEdges - Compute the pixels shared by two superpixel boundaries
% and the contour of the resulting shape
%
% [edge_pixels, contour_pixels] = SuperpixelEdges(sup_boundaries, adjmat, M, N)
% For every two adjacent superpixels, compute the pixels that are common to
% their boundaries and pixels on the contour of the resulting shape.
%
% Arguments:
%   sup_image - MxN superpixel image
%   sup_boundaries - num_sups x 1 cell array. Each element stores linear
%   indices of the boundary pixels for each superpixel
%   adjmat - num_sups x num_sups adjacency matrix
%   M,N  -  size of the image
%
% Returns:
%   edge_pixels - num_sups x num_sups cell array. Each element stores a
%   list of boundary pixels on the edge between two adjacent superpixels.
%   If not adjacent, the entry is empty.
%   contour_pixels - num_sups x num_sups cell array. Each element stores a
%   list of contour pixels on the contour of a union of two adjacent superpixels.
%   If not adjacent, the entry is empty.
%   adjmat - the fixed adjacency matrix
%
function [edge_pixels, contour_pixels, adjmat] = SuperpixelEdges(sup_image, sup_boundaries, adjmat)

    [M,N] = size(sup_image);
	num_sups = numel(sup_boundaries);
    edge_pixels = cell(num_sups, num_sups);
    contour_pixels = cell(num_sups, num_sups);
    
    [sup1, sup2] = find(tril(adjmat, -1));
    num_edges = numel(sup1);
    
    for i = 1:num_edges
        boundary1 = false(M,N);
        boundary1(sup_boundaries{sup1(i)}) = true;
        boundary2 = false(M,N);
        boundary2(sup_boundaries{sup2(i)}) = true;
        boundary1_dil = bwmorph(boundary1, 'dilate', 1);
        boundary2_dil = bwmorph(boundary2, 'dilate', 1);
        
        and_pixels = boundary1_dil & boundary2_dil;
        sup1_edge = find(and_pixels(sup_boundaries{sup1(i)}));
        sup2_edge = find(and_pixels(sup_boundaries{sup2(i)}));

        if (isempty(sup1_edge) || isempty(sup2_edge))
            adjmat(sup1(i), sup2(i)) = 0;
            adjmat(sup2(i), sup1(i)) = 0;
            continue;
        end
        
        num_sup1_edges = numel(sup_boundaries{sup1(i)});
        sup1_gap = sup1_edge(2:end) - sup1_edge(1:end-1);
        sup1_gap = [sup1_gap; num_sup1_edges - (sup1_edge(end) - sup1_edge(1))];
        
        num_sup2_edges = numel(sup_boundaries{sup2(i)});
        sup2_gap = sup2_edge(2:end) - sup2_edge(1:end-1);
        sup2_gap = [sup2_gap; num_sup2_edges - (sup2_edge(end) - sup2_edge(1))];
        [m, gap1_idx] = max(sup1_gap);
        [m, gap2_idx] = max(sup2_gap);
        gap1_start = sup1_edge(gap1_idx);
        gap1_end = sup1_edge(mod(gap1_idx, numel(sup1_edge))+1);
        gap2_start = sup2_edge(gap2_idx);
        gap2_end = sup2_edge(mod(gap2_idx, numel(sup2_edge))+1);
        
        if (gap1_start < gap1_end)
            gap1_pixels = sup_boundaries{sup1(i)}(gap1_start:gap1_end);
        else
            gap1_pixels = sup_boundaries{sup1(i)}([gap1_start:end,1:gap1_end]);
        end
        if (gap2_start < gap2_end)
            gap2_pixels = sup_boundaries{sup2(i)}(gap2_start:gap2_end);
        else
            gap2_pixels = sup_boundaries{sup2(i)}([gap2_start:end,1:gap2_end]);
        end
        
        
        edge_pixels{sup1(i), sup2(i)} = setdiff(sup_boundaries{sup1(i)}, gap1_pixels);
        edge_pixels{sup2(i), sup1(i)} = edge_pixels{sup1(i), sup2(i)};
        
        contour_pixels{sup1(i), sup2(i)} = [gap1_pixels; gap2_pixels];
        contour_pixels{sup2(i), sup1(i)} = contour_pixels{sup1(i), sup2(i)};
%         
%         b = zeros(M,N);
%         b(contour_pixels{sup1(i), sup2(i)}) = 1:numel(contour_pixels{sup1(i), sup2(i)});
%         imagesc(b);
%         pause;
    end